"use strict";
var DecoratorUtil = (function () {
    function DecoratorUtil() {
    }
    /**
     * create param decorators
     * @param value the value for the decorator
     * @param key the key under wich the value is saved
     * @returns the decorated function
     */
    DecoratorUtil.createParamDecorator = function (value, key) {
        return function (target, propertyKey, parameterIndex) {
            var existingDecorators = Reflect.getOwnMetadata(key, target, propertyKey) || [];
            existingDecorators.push({ paramName: value, index: parameterIndex });
            Reflect.defineMetadata(key, existingDecorators, target, propertyKey);
        };
    };
    /**
     * create decorators for the class or for a method
     * @param key he key under wich the value is saved
     * @param value the value for the decorator
     * @returns the decorated function
     */
    DecoratorUtil.decorateClassOrMethod = function (key, value) {
        return function (target, propertyKey, descriptor) {
            if (!propertyKey && !descriptor) {
                // add meta data to the class itself - e.g. target is the constructor and propertyKey and serviceDescription are undefined
                return Reflect.defineMetadata(key, value, target);
            }
            else {
                // add meta data to a function
                return Reflect.defineMetadata(key, value, descriptor.value);
            }
        };
    };
    return DecoratorUtil;
}());
exports.DecoratorUtil = DecoratorUtil;
//# sourceMappingURL=decoratorUtil.js.map